
###############################################################
####
#### HRD_indexes such as LST, HRD_LOH and NTAI calculation based on the ABSOLUTE ALLELE-SPECIFIC COPY NUMBER PROFILE 
####   obtained from SNP arrays of ~10^6 SNPs
####
#### COPY NUMBER PROFILE is supposed to be already cleaned from small (less 50 SNPs) CN Alterations and NAs
####
#### COPY NUMBER in each segment is characterized by the number of copies and major allele counts 
####
#### copy number profile is represented by the table with the following information:
#### "posSNPstart" (chr position of segment start) 
#### "posSNPend" (chr position of segment end)
#### "lenSNP" (the size on the segments in SNP counts)  
#### "chrArm" (ID of chromosome arm chrArm = chrN, for p arm and chrArm = chrN+0.5 for q arm  
####       no short arms for chromosomes 21 and 22; chromosomes X,XY,M are excluded
####
#### "copyNumber" (absolute copy number)
#### "majorAllele" (major allele counts), 1 for AB, 2 for ABB, 3 for ABBB, etc. LOH if copyNumber==majorAllele or majorAllele==0
####
####
##################################################################


### MAIN

	tmp<-readCopyNumberFile(segFileName="D:/CGH_Elisabetta/tmp_example.txt")

	tmp<-data.frame(tmp)	

	results<-HRD_indexes(tmp,minLengthSNP=500,LST_MB=10,LOH_MB=15)

	LST<-results$LST

	NTAI<-results$NTAI

	HRD_LOH<-results$HRD_LOH


### END MAIN



###############################################################
####
####  function HRD_indexes
####
#### tmp is data.frame with copy number profile
#### colnames = "posSNPstar" "posSNPend" "lenSNP" "chrArm" "copyNumber" "majorAllele"
#### minLengthSNP minimal size of the segment in SNP counts, usually 500 for Affymetix SNP6
#### LST_MB the size of LST to calculate
#### LOH_MB the size of LOH to calculate HRD_LOH
####
##################################################################


HRD_indexes<-function(tmp,minLengthSNP,LST_MB,LOH_MB){

LST<-NA
NTAI<-NA
HRD_LOH<-NA


if(tmp$posSNPstart[dim(tmp)[[1]]]!=0){ 
	
  
###### calculation of LST

	LST<-LST_counts(tmp, minLengthSNP, LST_MB)
	
###### calculation of NTAI

	NTAI<-NTAI_counts(tmp, minLengthSNP)
	
###### calculation of HRD_LOH

	HRD_LOH<-HRD_LOH_counts(tmp, minLengthSNP, LOH_MB)


}

##### output

out<-list(LST=LST,NTAI=NTAI,HRD_LOH=HRD_LOH)

}#



###############################################################
#####
##### Read the file with Copy Number data 
##### colnames to be there: "posSNPstar" "posSNPend" "lenSNP" "chrArm" "copyNumber" "majorAllele"
##### 
####################################################################

readCopyNumberFile<-function(segFileName){

	aa<-scan(segFileName,nlines=1,what="character",quiet = T)
	k<-1
	
	while((length(grep("#",aa))!=0)&&(grep("#",aa)==1)){
		aa<-scan(segFileName,skip=k,nlines=1,what="character",quiet = T)
		k<-k+1
	}
	
	tmp<-read.delim(segFileName,skip=k-1, header=T,sep="\t")
	tmp<-data.matrix(tmp)
	
	out<-tmp
}




###############################################################
#####
##### Calculation of Telomeric Allelic imbalance
##### NTAI_counts<-function(tmp,minLengthSNP)
#####
##### tmp is data.frame with copy number profile
##### minLengthSNP minimal size of the segment in SNP counts, usually 500 for Affymetix SNP6
#####
##### NTAI is the number of chromosome arms with Allelic imbalance at the telomer {}
##### 
####################################################################


NTAI_counts<-function(tmp,minLengthSNP){


######## elimination of segments with 0 copy number

	tt<-which(tmp$copyNumber==0)
	if(length(tt)>0){tmp<-tmp[-tt,]}
	

######## set copyNumber to 2 and majorAllele to 1 in all segments with ALLELIC BALANCE
######## set copyNumber to 1 and majorAllele to 1 in all segments with ALLELIC imBALANCE


	tt<-which(tmp$copyNumber-2*tmp$majorAllele==0)

	tmp$copyNumber<-1

	tmp$majorAllele<-1
	
	if(length(tt)>0){tmp$copyNumber[tt]<-2;tmp$majorAllele[tt]<-1}

	if(dim(tmp)[[1]]>1){

		for(k in 1:(dim(tmp)[[1]]-1)){

			str1<-as.numeric(c(tmp$chrArm[k],tmp$copyNumber[k],tmp$majorAllele[k]))
			str2<-as.numeric(c(tmp$chrArm[k+1],tmp$copyNumber[k+1],tmp$majorAllele[k+1]))
			
			if(sum(sign(abs(str1-str2)))==0){

				tmp$posSNPstart[k+1]<-tmp$posSNPstart[k]
				tmp$lenSNP[k+1]<-tmp$lenSNP[k+1]+tmp$lenSNP[k]
				tmp$posSNPstart[k]<-0
			}
		}#k
	
	}
	tt<-which(tmp$posSNPstart!=0)
	
	if(length(tt)<=1){ Ntai<-0
	
	}else{

		tmp<-tmp[tt,]

######## calculate NTAI with minimal segment size of minLengthSNP
	
		Ntai<-0

		nRows<-dim(tmp)[[1]]

		if((tmp$copyNumber[1]==1)&(tmp$lenSNP[1]>minLengthSNP)&(tmp$chrArm[2]==tmp$chrArm[1])){Ntai<-Ntai+1}

		if(nRows>2){
			for(k in 2:(nRows-1)){
				if(trunc(tmp$chrArm[k])!=trunc(tmp$chrArm[k-1])){
					if((tmp$copyNumber[k]==1)&(tmp$lenSNP[k]>minLengthSNP)&(tmp$chrArm[k+1]==tmp$chrArm[k])){Ntai<-Ntai+1}
				}
			}#k	
			
			for(k in 2:(nRows-1)){
				if(trunc(tmp$chrArm[k+1])!=trunc(tmp$chrArm[k])){
					if((tmp$copyNumber[k]==1)&(tmp$lenSNP[k]>minLengthSNP)&(tmp$chrArm[k-1]==tmp$chrArm[k])){Ntai<-Ntai+1}
				}
			}#k	
		}

		if((tmp$copyNumber[nRows]==1)&(tmp$lenSNP[nRows]>minLengthSNP)&(tmp$chrArm[nRows-1]==tmp$chrArm[nRows])){Ntai<-Ntai+1}

	}#any variation in CN
	
out<-Ntai

}#function
	


###############################################################
#####
##### HRD_LOH score calculation 
##### HRD_LOH_counts<-function(tmp,minLengthSNP=500,LOH_MB=15)
#####
##### tmp is data.frame with copy number profile
##### minLengthSNP minimal size of the segment in SNP counts, usually 500 for Affymetix SNP6
##### LOH_MB the size of HRD_LOH to calculate, usually, 15-16MB
#####
##### HRD_LOH is the number of chromosomes with LOH>LOH_MB and less than the chromosome length {}
#####
####################################################################


HRD_LOH_counts<-function(tmp,minLengthSNP,LOH_MB){

	
######## supress chromosome arm information, turn to chromosome 

	tmp$chrArm<-trunc(tmp$chrArm)

######## get the size of chromosomes

	tmp1<-tmp
	
	if(dim(tmp1)[[1]]>1){

		for(k in 1:(dim(tmp1)[[1]]-1)){

			if(tmp1$chrArm[k]==tmp1$chrArm[k+1]){
				tmp1$posSNPstart[k+1]<-tmp1$posSNPstart[k]
				tmp1$posSNPstart[k]<-0
			}
		}#k
	
	}

	tt<-which(tmp1$posSNPstart!=0)
	if(length(tt)>0){tmp1<-tmp1[tt,]}

	sizeChr_MB<-round((tmp1$posSNPend-tmp1$posSNPstart)/1000000,1)
	names(sizeChr_MB)<-tmp1$chrArm


######## eliminate the segments with 0 copy number

	tt<-which(tmp$copyNumber!=0)
	if(length(tt)>0){tmp<-tmp[tt,]}
	

######## set copyNumber to 2 and majorAllele to 1 in all segments with ROH (Retention of Heterozygosity)
######## set copyNumber to 1 and majorAllele to 1 in all segments with LOH (Loss of Heterozygosity)


	tt<-which(tmp$copyNumber-tmp$majorAllele==0 | tmp$majorAllele==0 | tmp$copyNumber>6 & (tmp$copyNumber-tmp$majorAllele)==1)

	tmp$copyNumber<-2
	tmp$majorAllele<-1

	if(length(tt)>0){tmp$copyNumber[tt]<-1;tmp$majorAllele[tt]<-1}


######## unite the segments with identical CN and MA status and filter segments smaller than minLengthSNP

	for(jj in 1:2){

		if(dim(tmp)[[1]]>1){

			for(k in 1:(dim(tmp)[[1]]-1)){

				if(tmp$chrArm[k]==tmp$chrArm[k+1] & tmp$copyNumber[k]==tmp$copyNumber[k+1]){
					
					tmp$posSNPstart[k+1]<-tmp$posSNPstart[k]
					tmp$lenSNP[k+1]<-tmp$lenSNP[k+1]+tmp$lenSNP[k]
					tmp$posSNPstart[k]<-(0)
				}	
			}#k
	
			tt<-which(tmp$posSNPstart!=0)
			tmp<-tmp[tt,]

			tt<-which(tmp$lenSNP>minLengthSNP)
			if(length(tt)>0){tmp<-tmp[tt,]}

		}
	}#jj
	

	if(dim(tmp)[[1]]<=1){ HRD_LOH<-0
	
	}else{

				
####### HRD LOH score
	
		tmp$lenSNP<-round((tmp$posSNPend-tmp$posSNPstart)/1000000,0)

		for(k in 1:length(sizeChr_MB)){

			cc<-which(tmp$chrArm==as.numeric(names(sizeChr_MB)[k]))

			if(length(cc)>0){tmp$majorAllele[cc]<-round(tmp$lenSNP[cc]/sizeChr_MB[k],2)}
		}#k

		tt<-which(tmp$lenSNP>=LOH_MB & tmp$copyNumber==1 & tmp$majorAllele<0.95)
	
		if(length(tt)>0){HRD_LOH<-length(table(tmp$chrArm[tt]))}else{HRD_LOH<-0}
						

	}# 

out<-HRD_LOH

}# function 



###################################################################
#####
#####   Calculation of LST 
##### LST_counts<-function(tmp,minLengthSNP,LST_MB)
#####
##### tmp is data.frame with copy number profile
##### minLengthSNP minimal size of the segment in SNP counts, usually 500 for Affymetix SNP6
##### LST_MB the size of LST to calculate
#####
##### LST is the number of CN breaks between segments of at least 10Mb each, with chromosomal distance between the segments less than 3 Mb
##### LSTs are counted after segments of 3Mb are removed
##### centromeric breaks are not counted {}
#####
####################################################################


LST_counts<-function(tmp,minLengthSNP,LST_MB){

LST<-0

###### eliminate segments of less than 3MB

tt<-which(round((tmp$posSNPend-tmp$posSNPstart)/10^6,1)>=3)

if(length(tt)>1){

	tmp<-tmp[tt,]

###### Merge segments with identical copyNumber and majorAllele counts if they are closer than 3MB

	for(k in 1:(dim(tmp)[[1]]-1)){

		if(round((tmp$posSNPstart[k+1]-tmp$posSNPend[k])/10^6,1)<3){

			str1<-as.numeric(c(tmp$chrArm[k],tmp$copyNumber[k],tmp$majorAllele[k]))
			str2<-as.numeric(c(tmp$chrArm[k+1],tmp$copyNumber[k+1],tmp$majorAllele[k+1]))
				
			if(sum(sign(abs(str1-str2)))==0){
				tmp$posSNPstart[k+1]<-tmp$posSNPstart[k]
				tmp$lenSNP[k+1]<-tmp$lenSNP[k+1]+tmp$lenSNP[k]
				tmp$posSNPstart[k]<-0
			}
		}
	}#k
	
	tt<-which(tmp$posSNPstart!=0)
	
	if(length(tt)<=1){ LST<-0
	
	}else{

		tmp<-tmp[tt,]

###### Identification of copy number breaks between intra-chrArm segments both of more than LST_MB in size 


		LST<-numeric(dim(tmp)[[1]])

		for(j in 2:length(LST)){

			if(tmp$chrArm[j]==tmp$chrArm[j-1] & round((tmp$posSNPstart[j]-tmp$posSNPend[j-1])/10^6,1)<3){

				if(tmp$lenSNP[j]>minLengthSNP & tmp$lenSNP[j-1]>minLengthSNP){
	
					if(round((tmp$posSNPend[j]-tmp$posSNPstart[j])/10^6,0)>=LST_MB & round((tmp$posSNPend[j-1]-tmp$posSNPstart[j-1])/10^6,0)>=LST_MB){

						if(tmp$copyNumber[j]!=tmp$copyNumber[j-1]){LST[j]<-1

						}else{if(tmp$majorAllele[j]!=tmp$majorAllele[j-1] & tmp$copyNumber[j,c_CN]<7){LST[j]<-1}}
					}
				}
			}
		}#j


	}#else to no change in copy number

}# if any segment >3Mb

out<-sum(LST)


}#function







